<?php
global $wpdb;
define('TABLE_VENUES', $wpdb->prefix . 'gigs_venue');
define('TABLE_GIGS', $wpdb->prefix . 'gigs_gig');
define('TABLE_PERFORMANCES', $wpdb->prefix . 'gigs_performance');

class dtcGigs {
	function generateSlug($x) {
		$x = strtolower($x);
		$x = str_replace(' ', '-', $x);
		$x = preg_replace('#[^a-z\-0-9]#', '', $x);
		return $x;
	}
		
	function escapeForInput($x) {
		if ( '0000-00-00' == $x ) {
			$x = '';
		}
		echo htmlspecialchars($x);
	}
	
	function dateFormat($date, $short = true) {
		$options = get_option('gigs-calendar');
		if ( $short ) {
			return date($options['short-date-format'], strtotime($date));
		} else {
			return date($options['long-date-format'], strtotime($date));
		}
	}
}

class dtc_gigs_baseAR {
	var $wpdb, $_rows, $_row, $_new = true, $_table, $_count;
	
	function dtc_gigs_baseAR($id = null){
		global $wpdb;
		$this->wpdb = $wpdb;
		if ( $id !== null ) {
			$this->get($id);
		}
	}
	
	function fetch() {
		$this->_row = each($this->_rows);
		if ( $this->_row ) {
			$this->load();
			return true;
		} else {
			return false;
		}
	}

	function generate_slug() {
		if ( isset($this->slug) ) {
			$first = true;
			$name = $this->_name;
			do {
				if ( $first ) {
					$this->slug = dtcGigs::generateSlug($this->$name);
					$first = false;
				} else {
					ereg('[0-9]*$', $this->slug, $match);
					$match = ((int) $match[0]) + 1;
					if ( 1 == $match ) $match++;
					$this->slug = preg_replace('/[0-9]*$/', '', $this->slug);
					$this->slug .= $match;
				}
				
			} while ( $this->wpdb->get_var('SELECT COUNT(*) FROM `' . $this->_table . '` WHERE slug = "' . $this->slug . '"') );
		}
	}

	
	function get($id) {
		$this->search('`id` = ' . (int) $id);
		return $this->fetch();
	}
	
	function getRow() {
		$row = array();
		foreach ( $this->_fields as $f ) {
			$row[$f]  = $this->$f;
		}
		return $row;
	}
	



	
	function load() {
		foreach ( $this->_fields as $f ) {
			$this->$f = stripslashes($this->_row['value']->$f);
		}
	}
	
	function search($where = null, $order = null) {
		$this->_new = false;
		$this->_rows = $this->wpdb->get_results('SELECT * FROM `' . $this->_table . '` ' . ($where !== null ? 'WHERE ' . $where : '') . ($order !== null ? ' ORDER BY ' . $order : ''));
		$this->_count = count($this->_rows);
		return count($this->_rows);
	}
	
	function save() {
		$first = true;
		$name = $this->_name;
		$this->generate_slug();
		if ( $this->_new ) {
			$fields = $this->_fields;
			$values = array();
			unset($fields[0]);
			
			$this->pre_insert();

			foreach ( $fields as $f ) {
				$values[$f] = '"' . $this->wpdb->escape(stripslashes($this->$f)) . '"';
			}
			
			if ( !$this->wpdb->query('INSERT INTO `' . $this->_table . '` (`' . implode('`, `', $fields) . '`) VALUES (' . implode(', ', $values) . ')') ) {
				return false;
			}
			$this->id = $this->wpdb->insert_id;

			$this->post_insert();
			$this->get((int) $this->id);

			return true;
		} else {
			$fields = $this->_fields;
			$values = array();
			unset($fields[0]);
			
			$this->pre_update();
			
			foreach ( $fields as $f ) {
				$values[$f] = '`' . $f . '` = "' . $this->wpdb->escape(stripslashes($this->$f)) . '"';
			}
			$result = (bool) $this->wpdb->query('UPDATE `' . $this->_table . '` SET ' . implode(', ', $values) . ' WHERE `id` = ' . (int) $this->id);
			
			if ( $result ) $this->post_update();
			
			$this->get((int) $this->id);
			
			return $result;
		}
	}
	
	function pre_insert(){}
	function post_insert(){}
	function pre_update(){}
	function post_update(){}
	function extraJSON() {return array();}
	
	function delete() {
		if ( !empty($this->id) ) {
			return (bool) $this->wpdb->query('DELETE FROM `' . $this->_table . '` WHERE `id` = ' . (int) $this->id);
		}
	}
	
	function toJSON($single = true) {
		if ( $single ) {
			$fields = array();
			if ( function_exists("json_encode") ) {
				foreach ( $this->_fields as $f ) {
					$fields[$f] = $this->$f;
				}
				
				$extra = $this->extraJSON();
				$fields = array_merge($fields, $extra);
				return json_encode($fields);
			} else {
				foreach ( $this->_fields as $f ) {
					$fields[] = '"' . $f . '":"' . str_replace("\n", '\n', addslashes($this->$f)) . '"';
				}
			
				$extra = $this->extraJSON();
				$fields = array_merge($fields, $extra);

				return '{' . implode(', ', $fields) . '}';
			}
		}
	}
}

class venue extends dtc_gigs_baseAR {
	var $_table = TABLE_VENUES, $_name = 'name';
	var $id, $name, $address, $city, $state, $country, $postalCode, $contact, 
		$phone, $email, $link, $notes, $private, $apiID, $deleted, $customMap;
	var $_fields = array(
		'id',
		'name',
		'address',
		'city',
		'state',
		'country',
		'postalCode',
		'contact',
		'phone',
		'email',
		'link',
		'notes',
		'private',
		'apiID',
		'deleted',
		'customMap',
	);
	
	function delete($forReal = false) {
		if ( $forReal ) {
			parent::delete();
		} else {
			$this->deleted = 1;
			$this->save();
		}
	}
	
	function getAddress($oneLine = false) {
		$a = array();
		$city = "";
		if ( $oneLine ) {
			if ( !empty($this->address) ) $a[] = str_replace("\n", ', ', $this->address);
			if ( !empty($this->city) ) $a[] = $this->city;
			if ( !empty($this->state) ) $a[] = $this->state;
			if ( !empty($this->postalCode) ) $a[] = $this->postalCode;
			if ( !empty($this->country) ) $a[] = $this->country;
			return implode(', ', $a);
		} else {
			if ( !empty($this->address) ) $a[] = $this->address;
			if ( !empty($this->city) ) $temp .= $this->city;
			if ( !empty($this->state) ) $temp .= (!empty($temp) ? ', ' : '' ) . $this->state;
			if ( !empty($this->country) ) $temp .= (!empty($temp) ? ', ' : '' ) . $this->country;
			if ( !empty($this->postalCode) ) $temp .= (!empty($temp) ? ' ' : '' ) . $this->postalCode;
			if ( !empty($temp) ) $a[] = $temp;
			return implode("\n", $a);
		}
	}
	
	function getCity() {
		$c = array();
		if ( !empty($this->city) ) $c[] = $this->city;
		if ( !empty($this->state) ) $c[] = $this->state;
		//if ( !empty($this->country) ) $c[] = $this->country;
		return implode(', ', $c);
	}
	
	function getMapLink() {
		global $options;
		if ( empty($this->customMap) ) {
			$a = array();
			
			if ( empty($this->address) ) 
				$a[] = $this->name . ' near:';
			else {}
				$a[] = str_replace("\n", ", ", $this->address);
			if ( !empty($this->city) ) $a[] = $this->city;
			if ( !empty($this->state) ) $a[] = $this->state;
			if ( !empty($this->country) ) $a[] = $this->country;
			if ( !empty($this->postalCode) ) $a[] = $this->postalCode;
			return 'http://maps.google.com/?q=' . urlencode(str_replace('near:,', 'near:', implode(', ', $a)));
		} else {
			if ( preg_match("/^https?:\/\//", $this->customMap) ) {
				return $this->customMap;
			} else {
				return 'http://maps.google.com/?q=' . urlencode($this->customMap);
			}
		}
	}
	
	function extraJSON() {
		if ( function_exists("json_encode") ) {
			$fields['mapLink'] = $this->getMapLink();
		} else {
			$fields[] = '"mapLink":"' . str_replace("\n", '\n', addslashes($this->getMapLink())) . '"';
		}
		return $fields;
	}
}

class gig extends dtc_gigs_baseAR {
	var $_table = TABLE_GIGS, $_name = 'id';
	var $id, $venueID, $date, $notes, $postID, $eventName;
	var $_fields = array(
		'id',
		'venueID',
		'date',
		'notes',
		'postID',
		'eventName',
	);

	function delete() {
		$p = $this->getPerformances();
		while ( $p->fetch() ) {
			$p->delete();
		}
		wp_delete_post($this->postID);
		parent::delete();
	}
	
	function getByPostID($postID) {
		$this->search("postID = " . (int) $postID);
		return $this->fetch();
	}
	
	function getPermalink() {
		$post = get_permalink($this->postID);
		return $post;
	}
	
	function getPerformances() {
		$p = new performance();
		if ( $this->id ) {	
			$p->search("`gigID` = " . (int) $this->id, "`time`");
		}
		return $p;
	}
	
	function getVenue() {
		$v = new venue();
		if ( $this->venueID ) {
			$v->get($this->venueID);
		}
		return $v;
	}
	
	function pre_insert() {
		global $options, $wpdb;
		$user = wp_get_current_user();
		$v = $this->getVenue();
		$title = array();
		foreach ( $options['gig-title-show'] as $f ) {
			switch ( $f ) {
				case 'city' :
					$title[] = $v->getCity();
					break;
				case 'eventName' :
					$title[] = $this->eventName;
					break;
				case 'venue' :
					$title[] = $v->name;
					break;
				case 'date' :
					$title[] = dtcGigs::dateFormat($this->date);
					break;
			}
		}
		
		foreach ( $title  as $key => $value ) {
			if ( empty($value) ) unset($title[$key]);
		}

		$id = wp_insert_post(array(
			'post_author'		=> $user->ID,
			'post_title'		=> $wpdb->escape(implode(' - ', $title)),
			'post_date'			=> $wpdb->escape($this->date),
			'post_date_gmt'		=> date('Y-m-d H:i:s', strtotime($this->date) - (60 * 60 * get_option('gmt_offset'))),
			'post_status'		=> 'publish',
			'post_type' 		=> 'post',
			'post_category'		=> array($options['category']),
		));
		$this->postID = $id;
	}
	
	function pre_update() {
		global $options, $wpdb;
		$v = $this->getVenue();
		
		$title = array();
		foreach ( $options['gig-title-show'] as $f ) {
			switch ( $f ) {
				case 'city' :
					$title[] = $v->getCity();
					break;
				case 'eventName' :
					$title[] = $this->eventName;
					break;
				case 'venue' :
					$title[] = $v->name;
					break;
				case 'date' :
					$title[] = dtcGigs::dateFormat($this->date);
					break;
			}
		}
		
		foreach ( $title  as $key => $value ) {
			if ( empty($value) ) unset($title[$key]);
		}

		
		$post = get_post($this->postID);
		
		$post->post_date = $wpdb->escape($this->date);
		$post->post_date_gmt = date('Y-m-d H:i:s', strtotime($this->date) - (60 * 60 * get_option('gmt_offset')));
		$post->post_title = $wpdb->escape(implode(' - ', $title));
		$post->post_name = "";
		$post->post_status = "publish"; // Do this to trick WordPress into resetting the GUID.
		$post->post_category = array($options['category']);

		wp_update_post($post);
	}
	
	function post_insert() {
		$post = get_post($this->postID);
		$post->post_status = "publish";
		wp_update_post($post);
	}

	function extraJSON() {
		$post = get_post($this->postID);
		$v = $this->getVenue();
		$p = $this->getPerformances();
		if ( function_exists("json_encode") ) {
			$fields['permalink'] = $this->getPermalink();
			$fields['venue'] = $v->getRow();
			while ( $p->fetch() ) {
				$fields['performances'][] = $p->getRow();
			}
		} else {
			$fields[] = '"permalink":"' . str_replace("\n", '\n', addslashes($this->getPermalink())) . '"';
			$ps = array();
			$fields[] = '"venue":' . $v->toJSON();
			while ( $p && $p->fetch() ) {
				$ps[] = $p->toJSON();
			}
			$fields[] = '"performances":[' . implode(', ', $ps) . ']';
		}
		return $fields;
	}
}

class performance extends dtc_gigs_baseAR {
	var $_table = TABLE_PERFORMANCES, $_name = 'id';
	var $id, $gigID, $time = '20:00', $link, $shortNotes, $ages = 'All Ages';
	var $_fields = array(
		'id',
		'gigID',
		'time',
		'link',
		'shortNotes',
		'ages',
	);
}

?>
